/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

package fr.ill.ics.cameo.test;

import fr.ill.ics.cameo.Application;
import fr.ill.ics.cameo.RemoteException;
import fr.ill.ics.cameo.messages.Message;


public class TestResponderApplication {

	public static void main(String[] args) {

		Application.This.init(args);
		
		if (Application.This.isAvailable()) {
			System.out.println("connected");
		}
		
		try {
			System.out.println("creating responder");
			
			// Create the responder.
			Application.Responder responder = Application.Responder.create("responder");
			
			// Set the state.
			Application.This.setRunning();
			
			
			// Receive the first request.
			Application.Request request = responder.receive();
			System.out.println("received request " + request.get());

			// Reply.
			request.reply("1st response");
			request.terminate();
			
			// Receive the second request.
			request = responder.receive();
			
			byte[][] data = request.getTwoBinaryParts();
			System.out.println("received request " + Message.parseString(data[0]) + " " + Message.parseString(data[1]));

			// Reply.
			request.reply("2nd response");
			
			
			// Receive the third request.
			request = responder.receive();
			System.out.println("received request " + request.get());

			// Reply.
			//request.setTimeout(100);
			request.reply("3rd response");
			request.terminate();
			
			
			// Receive the fourth request.
			request = responder.receive();
			System.out.println("received request " + request.get());

			// Reply.
			request.reply("4th response");
			request.terminate();
			
			
			// Test connection.
			Application.Instance requester = request.connectToRequester();
			System.out.println("requester is " + requester);
						
			request.terminate();
			
		} catch (RemoteException e) {
			System.out.println("responder error");
			
		} finally {
			// Do not forget to terminate This.
			Application.This.terminate();			
		}
		
		System.out.println("finished the application");
	}

}