/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

package fr.ill.ics.cameo.test;

import fr.ill.ics.cameo.Application;
import fr.ill.ics.cameo.Server;


public class TestStop {

	public static void main(String[] args) {

		Application.This.init(args);
		Server server = Application.This.getServer();
		
		int numberOfTimes = 1;
		if (args.length > 1) {
			numberOfTimes = Integer.parseInt(args[0]);
		}
		
		if (Application.This.isAvailable() && server.isAvailable()) {
			System.out.println("application connected");
		} else {
			System.exit(-1);
		}
		
		try {
			// loop the number of times.
			for (int i = 0; i < numberOfTimes; ++i) {
				
				// start the application
				Application.Instance stopApplication = server.start("stopjava");
	
				System.out.println("waiting 100ms...");
				// stop the application after 1s
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
				}
				
				System.out.println("stopping application " + stopApplication.getNameId());
				stopApplication.stop();
				
				// the getString is blocking until the application finishes
				String result = stopApplication.getStringResult();
				
				if (result != null) {
					System.out.println("stop application returned " + result);

				} else {
					System.out.println("stop application has no result");
				}
				
				System.out.println("stop application finished");
			}
						
		} finally {
			// do not forget to terminate the server and application
			Application.This.terminate();
		}
		
		System.out.println("finished the application");
	}

}