/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <cameo/cameo.h>

using namespace std;
using namespace cameo;

int main(int argc, char *argv[]) {

	/**
	 * This goal of the test was to test the linger value. In fact, it does not seem important for us.
	 *
	 *
	 *

	application::This::init(argc, argv);

	// New block to ensure cameo objects are terminated before the application.
	{
		string applicationName;

		int numberOfTimes = 1;

		if (argc > 2) {
			applicationName = argv[1];
			cout << "requester application is " + applicationName << endl;

			if (argc > 3) {
				numberOfTimes = stoi(argv[2]);
			}

		} else {
			cerr << "arguments: [application name] <number of times>" << endl;
			return -1;
		}

		cout << "number of times is " << numberOfTimes << endl;

		// get the client services
		Server& server = application::This::getServer();

		if (application::This::isAvailable() && server.isAvailable()) {
			cout << "connected server " << server << endl;
		}

		// create a responder to the application applicationName
		unique_ptr<application::Responder> responder = application::Responder::create("responder");

		cout << "created responder " << *responder << endl;

		if (responder.get() == 0) {
			cout << "responder error" << endl;
			return -1;
		}

		int base = 10;

		vector<int> timeouts = {-2, base}; // -2 => 0
		vector<bool> lingers = {false, true};

		// loop the number of times.
		for (int i = 0; i < numberOfTimes; ++i) {

			for (int t : timeouts) {
				for (bool l : lingers) {
					for (int j = 0; j < 2; ++j) {

						int r = 0;
						switch (j) {
						case 0: r = t / 2; break;
						case 1: r = t + base; break;
						}
						if (r < 0) r = base;

						cout << "t = " << t << ", l = " << l << ", r = " << r << endl;

						vector<string> args = {to_string(r)};

						// start the application.
						unique_ptr<application::Instance> requesterApplication = server.start(applicationName, args);

						cout << "started application " << *requesterApplication << endl;

						// receive request.
						unique_ptr<application::Request> request = responder->receive();
						cout << "received request " << *request << endl;

						request->setTimeout(t, l);

						string response = string("response ") + to_string(r);
						request->reply(response);

						this_thread::sleep_for(chrono::milliseconds(2*r));

						requesterApplication->kill();
						application::State state = requesterApplication->waitFor();
						cout << *requesterApplication << " terminated with state " << application::toString(state) << endl << endl;
					}
				}
			}
		}

		cout << "finished the application" << endl;
	}

 */

	return 0;
}
