/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <cameo/cameo.h>

using namespace std;
using namespace cameo;

int main(int argc, char *argv[]) {

	application::This::init(argc, argv);

	// New block to ensure cameo objects are terminated before the application.
	{
		if (application::This::isAvailable()) {
			cout << "connected" << endl;
		}

		unique_ptr<application::Responder> responder;

		try {
			cout << "creating responder" << endl;

			responder = application::Responder::create("responder");

		} catch (const ResponderCreationException& e) {
			cout << "responder error" << endl;
			return -1;
		}

		application::This::setRunning();

		while (true) {

			unique_ptr<application::Request> request = responder->receive();
			cout << "received request " << *request << endl;

			request->reply("done");

			this_thread::sleep_for(chrono::milliseconds(10));
		}

		cout << "finished the application" << endl;
	}

	return 0;
}
